function   blpsample1
% Best lead per sample, 105 records annotated by one cardiologist
%
% Author: Qinghua Zhang
% Copyright 2005 INRIA

%Data directory path
QTdir = qtdatapath;

% Get the list of data file names
dst = dir([QTdir, '*.Tendq1c']);
fichcells = {dst.name};
nfich = length(fichcells);
dnames = cell(nfich,1);
for kd=1:nfich
  dnames{kd} = fichcells{kd}(1:end-8);
end

errcell1 = cell(nfich,1);

% Memory initialization
meanvec1 = zeros(nfich,1);
stdvec1 = zeros(nfich,1);

ncycvec1 = zeros(nfich,1);
 
for kd=1:length(dnames)
  dataname = dnames{kd};
  disp(['Processing record number ', num2str(kd), ', file name: ', dataname])
  
  if (strcmp(dataname, 'sel35') | strcmp(dataname, 'sel37'))
    meanvec1(kd) = 0;
    stdvec1(kd) = 0;
    ncycvec1(kd) = 0;
    continue
  end
 
  [annodata, Tendindq1c] = DataConvert(dataname, QTdir);
  errcell1{kd} = CompareWith2Cardios(annodata, Tendindq1c);
  
  meanvec1(kd) = mean(errcell1{kd});
  stdvec1(kd) = std(errcell1{kd}); 
  ncycvec1(kd) = length(errcell1{kd});
 
end

uerr1 = cell2mat(errcell1);
umean1 = mean(uerr1)*4;
ustd1 = std(uerr1)*4;

mmean1 = mean(meanvec1)*4;
mstd1 = mean(stdvec1)*4;

wmean1 = 4*sum(meanvec1.*ncycvec1)/sum(ncycvec1);
wstd1 = 4*sum(stdvec1.*ncycvec1)/sum(ncycvec1);

ncyc1 = sum(ncycvec1);

disp(['Auto to Cardio1,', num2str(ncyc1) , ' beats.', ' Result: ', num2str(mmean1), '+/-', num2str(mstd1), ...
      '  weighted: ', num2str(wmean1), '+/-', num2str(wstd1)])
disp(['Auto to Cardio1, unique set: ', num2str(umean1), '+/-', num2str(ustd1)])
    
%Auto to Cardio1,3542 beats. Result: 0.31214+/-17.4331  weighted: 0.37945+/-18.0463
%Auto to Cardio1, unique set: 0.37945+/-38.3476
%Elapsed time is 296.079619 seconds.

%========================================
function [annodata, Tendindq1c] = DataConvert(dataname, QTdir)
% data convertion
% annodata: annotated portion of data
% Tendindq1c: T-wave end indices annotated by q1c 

alldata = load([QTdir, dataname, '.txt']);

fid = fopen([QTdir, dataname, '.Tendq1c']);
timecol = textscan(fid,'%s%f%*[^\n]');
fclose(fid);
Tendindq1c = timecol{2};    

% Determine beginning and end of annotated data
Tendinddebut =  Tendindq1c(1);
Tendindfin = Tendindq1c(end);
dataind1 = find(round(alldata(:,1)*250)==Tendinddebut);
leftmargin = 1200;  
if dataind1>leftmargin
  dataind1 = dataind1-leftmargin;
else
  leftmargin = dataind1-1;
  dataind1 = 1;
end
dataind2 = find(round(alldata(:,1)*250)==Tendindfin);  
dataind2 = min(dataind2+1200, size(alldata,1));

% Shift Tendindq1c in agreement with annodata
Tendindq1c = Tendindq1c -(Tendinddebut-leftmargin) + 1;

annodata = alldata(dataind1:dataind2,:);    

%================
function err1 = CompareWith2Cardios(annodata, Tendindq1c)
% Compare automatically located T-wave ends with Tendindq1c, Tendindq2c

fs = 250; % sampling frequency

% compute T-wave ends
Tend0 = twaveend(annodata(:,2), fs);
Tend1 = twaveend(annodata(:,3), fs);

%compare with Tendindq1c
nTend = length(Tendindq1c);
Tcmp1 = zeros(nTend,1);
for ks = 1:nTend
  [dum, kc] = min(abs(Tend0-Tendindq1c(ks)));
  Tq1d0 = Tend0(kc);
  [dum, kc] = min(abs(Tend1-Tendindq1c(ks)));
  Tq1d1 = Tend1(kc);
  if abs(Tq1d0-Tendindq1c(ks))<abs(Tq1d1-Tendindq1c(ks))
    Tcmp1(ks) = Tq1d0;
  else
    Tcmp1(ks) = Tq1d1;
  end
  
end

err1 = Tcmp1-Tendindq1c;


