function  blpsample2
% Best lead per sample, 11 records annotated by two cardiologists
%
% Author: Qinghua Zhang
% Copyright 2005 INRIA

%Data directory path
QTdir = qtdatapath;

% Get the list of data file names
dst = dir([QTdir, '*.Tendq2c']);
fichcells = {dst.name};
nfich = length(fichcells);
dnames = cell(nfich,1);
for kd=1:nfich
  dnames{kd} = fichcells{kd}(1:end-8);
end

errcell1 = cell(nfich,1);
errcell2 = cell(nfich,1);
errCardioscell = cell(nfich,1);

% Memory initialization
meanvec1 = zeros(nfich,1);
stdvec1 = zeros(nfich,1);
meanvec2 = zeros(nfich,1);
stdvec2 = zeros(nfich,1);

ncycvec1 = zeros(nfich,1);
ncycvec2 = zeros(nfich,1);

meancardiosvec = zeros(nfich,1);
stdcardiosvec = zeros(nfich,1);

for kd=1:length(dnames)
  dataname = dnames{kd};
  disp(['Processing record number ', num2str(kd), ', file name: ', dataname])
  
  [annodata, Tendindq1c, Tendindq2c] = DataConvert(dataname, QTdir);
  [errcell1{kd}, errcell2{kd}, errCardioscell{kd}]  = ...
              CompareWith2Cardios(annodata, Tendindq1c, Tendindq2c);
  
  meanvec1(kd) = mean(errcell1{kd});
  stdvec1(kd) = std(errcell1{kd});
  meanvec2(kd) = mean(errcell2{kd});
  stdvec2(kd) = std(errcell2{kd});
  
  ncycvec1(kd) = length(errcell1{kd});
  ncycvec2(kd) = length(errcell2{kd});
  
  meancardiosvec(kd) = mean(errCardioscell{kd});
  stdcardiosvec(kd) = std(errCardioscell{kd});

end

uerr1 = cell2mat(errcell1);
umean1 = mean(uerr1)*4;
ustd1 = std(uerr1)*4;

uerr2 = cell2mat(errcell2);
umean2 = mean(uerr2)*4;
ustd2 = std(uerr2)*4;

uerr12 = cell2mat(errCardioscell);
umean12 = mean(uerr12)*4;
ustd12 = std(uerr12)*4;

mmean1 = mean(meanvec1)*4;
mstd1 = mean(stdvec1)*4;

mmean2 = mean(meanvec2)*4;
mstd2 = mean(stdvec2)*4;

wmean1 = 4*sum(meanvec1.*ncycvec1)/sum(ncycvec1);
wstd1 = 4*sum(stdvec1.*ncycvec1)/sum(ncycvec1);

wmean2 = 4*sum(meanvec2.*ncycvec2)/sum(ncycvec2);
wstd2 = 4*sum(stdvec2.*ncycvec2)/sum(ncycvec2);

ncyc1 = sum(ncycvec1);
ncyc2 = sum(ncycvec2);

mm12 = mean(meancardiosvec)*4;
ms12 = mean(stdcardiosvec)*4;
wm12 = 4*sum(meancardiosvec.*ncycvec2)/sum(ncycvec2);
ws12 = 4*sum(stdcardiosvec.*ncycvec2)/sum(ncycvec2);

disp(['Auto to Cardio1,', num2str(ncyc1) , ' beats.', ' Result: ', num2str(mmean1), '+/-', num2str(mstd1), ...
      '  weighted: ', num2str(wmean1), '+/-', num2str(wstd1)])
disp(['Auto to Cardio2,', num2str(ncyc2) , ' beats.', ' Result: ', num2str(mmean2), '+/-', num2str(mstd2), ...
      '  weighted: ', num2str(wmean2), '+/-', num2str(wstd2)]) 
disp(['Between two cardios: ', num2str(mm12), '+/-', num2str(ms12), '  weighted: '  num2str(wm12), '+/-', num2str(ws12)])

disp(['Auto to Cardio1, unique set: ', num2str(umean1), '+/-', num2str(ustd1)])
disp(['Auto to Cardio2, unique set: ', num2str(umean2), '+/-', num2str(ustd2)]) 
disp(['Between two cardios, unique set: ', num2str(umean12), '+/-', num2str(ustd12)]) 
    
%Auto to Cardio1,487 beats. Result: -7.4727+/-17.1778  weighted: -6.4476+/-18.9413
%Auto to Cardio2,402 beats. Result: -7.469+/-17.5093  weighted: -9.8209+/-17.0647
%Between two cardios: -2.1471+/-22.4177  weighted: -5.8408+/-24.4569
%Auto to Cardio1, unique set: -6.4476+/-20.9563
%Auto to Cardio2, unique set: -9.8209+/-22.427
%Between two cardios, unique set: -5.8408+/-39.8921
%Elapsed time is 29.328895 seconds.

%========================================
function [annodata, Tendindq1c, Tendindq2c] = DataConvert(dataname, QTdir)
% data convertion
% annodata: annotated portion of data
% Tendindq1c, Tendindq2c: T-wave end indices annotated by q1c and q2c

alldata = load([QTdir, dataname, '.txt']);

%Tend12 = load([QTdir, dataname, '.Tendq12c']);
%Tendindq2c = Tend12(:,2);

fid = fopen([QTdir, dataname, '.Tendq1c']);
timecol = textscan(fid,'%s%f%*[^\n]');
fclose(fid);
Tendindq1c = timecol{2};  

fid = fopen([QTdir, dataname, '.Tendq2c']);
timecol = textscan(fid,'%s%f%*[^\n]');
fclose(fid);
Tendindq2c = timecol{2};  

% Determine beginning and end of annotated data
Tendinddebut = min(Tendindq1c(1),Tendindq2c(1));
Tendindfin = max(Tendindq1c(end),Tendindq2c(end));
dataind1 = find(round(alldata(:,1)*250)==Tendinddebut);
leftmargin = 1200;  
if dataind1>leftmargin
  dataind1 = dataind1-leftmargin;
else
  leftmargin = dataind1-1;
  dataind1 = 1;
end
dataind2 = find(round(alldata(:,1)*250)==Tendindfin);  
dataind2 = min(dataind2+1200, size(alldata,1));

% Shift Tendindq1c, Tendindq2c in agreement with annodata
Tendindq1c = Tendindq1c -(Tendinddebut-leftmargin) + 1;
Tendindq2c = Tendindq2c -(Tendinddebut-leftmargin) + 1;

annodata = alldata(dataind1:dataind2,:);    

%================
function [err1, err2, errCardios]  = CompareWith2Cardios(annodata, Tendindq1c, Tendindq2c)
% Compare automatically located T-wave ends with Tendindq1c, Tendindq2c

fs = 250; % sampling frequency

% compute T-wave ends
Tend0 = twaveend(annodata(:,2), fs);
Tend1 = twaveend(annodata(:,3), fs);

%compare with Tendindq1c
nTend = length(Tendindq1c);
Tcmp1 = zeros(nTend,1);
for ks = 1:nTend
  [dum, kc] = min(abs(Tend0-Tendindq1c(ks)));
  Tq1d0 = Tend0(kc);
  [dum, kc] = min(abs(Tend1-Tendindq1c(ks)));
  Tq1d1 = Tend1(kc);
  if abs(Tq1d0-Tendindq1c(ks))<abs(Tq1d1-Tendindq1c(ks))
    Tcmp1(ks) = Tq1d0;
  else
    Tcmp1(ks) = Tq1d1;
  end
  
end

%compare with Tendindq2c
nTend = length(Tendindq2c);
Tcmp2 = zeros(nTend,1);
for ks = 1:nTend
  [dum, kc] = min(abs(Tend0-Tendindq2c(ks)));
  Tq1d0 = Tend0(kc);
  [dum, kc] = min(abs(Tend1-Tendindq2c(ks)));
  Tq1d1 = Tend1(kc);
  if abs(Tq1d0-Tendindq2c(ks))<abs(Tq1d1-Tendindq2c(ks))
    Tcmp2(ks) = Tq1d0;
  else
    Tcmp2(ks) = Tq1d1;
  end
end

err1 = Tcmp1-Tendindq1c;
err2 = Tcmp2-Tendindq2c;

% Errors between the two cardiologists
nTend = length(Tendindq2c);
errCardios = zeros(nTend,1);
for ks = 1:nTend
  [dum, kc] = min(abs(Tendindq1c-Tendindq2c(ks)));
  errCardios(ks) = Tendindq1c(kc)-Tendindq2c(ks); 
end 


